using System.ComponentModel;
using System.Reflection;
using IDIS.Models;
using Newtonsoft.Json;

namespace IDIS.Model
{
	public record IDIS_Model
	{
		[JsonIgnore]
		public virtual string[] Tags { get; set; }  = Array.Empty<string>();
		public virtual IDIS_Register_Data.ValueInfo[] AsValueInfo()
		{
			var list = new List<IDIS_Register_Data.ValueInfo>();
			list.Add(new IDIS_Register_Data.ValueInfo
			{
				Data = new IDIS_Register_Data.ValueData
				{
					Value = string.Join(";",Tags),
					Format = "string"
				},
				Index = 0,
				Type = "Tags"
			});
			var index = 1;
			foreach (var x in GetProperties())
			{
				var valueInfo = new IDIS_Register_Data.ValueInfo
				{
					Data =new IDIS_Register_Data.ValueData()
					{
						Value = x.GetValue(this)!.ToString()!,
						Format = x.PropertyType.Name,
					},
					Index = index++,
					Type = x.Name
				};
				list.Add(valueInfo);
			}
			return list.ToArray();
		}
		public virtual IDIS_Template AsTemplate()
		{
			var template = new IDIS_Template
			{
				Description = "This is a template",
				Prefix = "88.123.99",
				Version = "1.0.0"
			};

			var metaList = new List<IDIS_Template_Item>
			{
				new()
				{
					IdType = "tags",
					Name = "标签",
					MetaData = new IDIS_Template_Item_MetaData()
					{
						MinLength = 0,
						MaxLength = 255,
						Type = "string"
					}
				}
			};

			foreach (var x in GetProperties())
			{
				var item = new IDIS_Template_Item
				{
					IdType = x.Name,
				};
				var meta = new IDIS_Template_Item_MetaData();
				if (x.GetCustomAttribute<DisplayNameAttribute>() is { } displayName)
				{
					item.Name = displayName.DisplayName;
				}
				meta.Type = x.PropertyType.Name;
				meta.MinLength = 0;
				meta.MaxLength = 255;
				
				
				item.MetaData = meta;
				
				metaList.Add(item);
			}
			
			template.Items = metaList.ToArray();
			return template;
		}
		private PropertyInfo[] GetProperties()
		{
			return GetType()
				.GetProperties(BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic)
				.Where(x => x.Name is not ("EqualityContract" or "Tags"))
				.ToArray();
		}
	}
	
}