using System;
using IDIS.Models;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace IDIS
{
	public class IDIS_Response
	{
		public static IDIS_Response CreateFromJson(string json)
		{
			var value = new IDIS_Response();
			var jObject = JObject.Parse(json);
			if(jObject.TryGetValue(nameof(Message), out var message))
				value.Message = message.Value<string>();
			if(jObject.TryGetValue(nameof(Status), out var status))
				value.Status = status.Value<int>();
			value.jObject = jObject;
			return value;
		}
		public static IDIS_Response CreateFromJsonAs<T>(string json)
		{
			var value = CreateFromJson(json);
			try
			{
				if (value.jObject is not null && value.Status is 1 && value.jObject.TryGetValue("data", out var data))
				{
					value.Data = data.ToObject<T>();
				}
			}
			catch (Exception)
			{
				return value;
			}
			return value; 
		}
		public IDIS_Response()
		{
			
		}
		public IDIS_Response(string message)
		{
			Message = message;
		}

		public IDIS_Response(object data)
		{
			Data = data;
		}
		public IDIS_Response(string message, object data)
		{
			Message = message;
			Data = data;
		}
		public IDIS_Response(string message,int status, object data)
		{
			Message = message;
			Status = status;
			Data = data;
		}
		public IDIS_Response(string message,Enum status, object data)
		{
			Message = message;
			Status =(int)Convert.ChangeType(status,typeof(int));
			Data = data;
		}

        /// <summary>
        /// 信息
        /// </summary>
		[JsonProperty("message")]
		public string Message { get; set; } = "success";
		
		[JsonProperty("status")]
		public int Status { get; set; } = 1;

		[JsonProperty("data")]
		public object Data { get; set; } = true;

		[JsonIgnore] private JObject jObject;

		public T As<T>()
		{
			if (Data is T value)
			{
				return value;
			}
			throw new InvalidOperationException("Data is not of type T");
		}
		public bool TryAs<T>(out T value)
		{
			switch (Data)
			{
				case T _value:
					value = _value;
					return true; 
				case null when jObject?.TryGetValue("data", out var data) is true:
					value = data.Value<T>();
					return true;
				default:
					value = default!;
					return false;
			}
		}
	}
}
