using System;
using System.Collections.Generic;
using Cysharp.Threading.Tasks;
using IDIS.Models;

namespace IDIS.Services
{
	/// <summary>
	/// 标识解析节点的接口定义,包括了:增删改查,连接状态的回调
	/// </summary>
	public interface IDIS_Service
	{
		/// <summary>
		/// 已连接到服务的回调
		/// </summary>
		event Action OnConnected;
		
		/// <summary>
		/// 连接失败的回调
		/// </summary>
		event Action<string> OnConnectFailure;
		
		/// <summary>
		/// 断开连接的回调
		/// </summary>
		event Action OnDisconnected;

		/// <summary>
		/// 解析标识的回调的回调
		/// </summary>
		event Action<IDIS_Query_Data> OnQuery;
		
		/// <summary>
		/// 注册标识的回调
		/// </summary>
		event Action<string,string> OnRegister;

		/// <summary>
		/// 基于标识模板注册标识
		/// </summary>
		/// <param name="handle">标识码</param>
		/// <param name="templateVersion">标识模板</param>
		/// <param name="value">值,通常为字典</param>
		UniTask RegisterAsync(string handle, string templateVersion,IDIS_Register_Data.ValueInfo[] value);
		
		/// <summary>
		/// 异步删除标识
		/// </summary>
		/// <param name="code">标识码</param>
		/// <returns>是否删除成功</returns>
		UniTask DeleteAsync(string code);

		/// <summary>
		/// 异步删除标识
		/// </summary>
		/// <param name="code">标识码</param>
		/// <param name="index">标识索引</param>
		/// <returns></returns>
		UniTask DeleteIdDataAsync(string code, int index);
		
        /// <summary>
        /// 更新标识值
        /// </summary>
        /// <param name="code">标识码</param>
        /// <param name="value">需要更新的标识列表</param>
        /// <returns></returns>
		UniTask UpdateAsync(string code, IDictionary<string,object> value);
		
		/// <summary>
		/// 异步解析标识
		/// </summary>
		/// <param name="code">标识码</param>
		/// <returns><see cref="IDIS_Response"/></returns>
		UniTask<IDIS_Query_Data> QueryAsync(string code);
		
		/// <summary>
		/// 标识码是否已注册
		/// </summary>
		/// <param name="code">标识码</param>
		/// <returns>是否已注册</returns>
		UniTask<bool> IsExistAsync(string code);
	}
}